@@ -27,7 +27,7 @@ let { Goptions.get = ground_depth } =
     ()
 
 let default_intuition_tac =
-  let tac _ _ = Auto.h_auto None [] (Some []) in
+  let tac _ _ = Auto.gen_auto None [] (Some []) in
   let name = { Tacexpr.mltac_plugin = "coq-core.plugins.firstorder"; mltac_tactic = "auto_with"; } in
   let entry = { Tacexpr.mltac_name = name; mltac_index = 0 } in
   Tacenv.register_ml_tactic name [| tac |];
@@ -56,7 +56,11 @@ let () = Vernacextend.static_vernac_extend ~plugin:(Some "coq-core.plugins.first
                                                              ) in fun t
                                                              ?loc ~atts ()
                                                              -> coqpp_body t
-                                                             (Attributes.parse Tactic_option.tac_option_locality atts)), None))]
+                                                             (Attributes.parse 
+                                                             
+# 51 "plugins/firstorder/g_ground.mlg"
+                 Tactic_option.tac_option_locality
+                                                              atts)), None))]
 
 let () = Vernacextend.static_vernac_extend ~plugin:(Some "coq-core.plugins.firstorder") ~command:"Firstorder_Print_Solver" ~classifier:(fun _ -> Vernacextend.classify_as_query) ?entry:None 
          [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Print", 
@@ -212,7 +216,7 @@ let () = Tacentries.tactic_extend "coq-core.plugins.firstorder" "firstorder" ~le
         gen_ground_tac true (Option.map (tactic_of_value ist) t) l l' 
           )))]
 
-let () = Tacentries.tactic_extend "coq-core.plugins.firstorder" "gintuition" ~level:0 
+let () = Tacentries.tactic_extend "coq-core.plugins.firstorder" "gintuition" ~level:0 ~deprecation:( Deprecation.make ~since:"8.20" () ) 
          [(Tacentries.TyML (Tacentries.TyIdent ("gintuition", Tacentries.TyArg (
                                                               Extend.TUopt (
                                                               Extend.TUentry (Genarg.get_arg_tag wit_tactic)), 
