@@ -18,8 +18,7 @@ open Pltac
 
 
 
-let __coq_plugin_name = "funind_plugin"
-let _ = Mltop.add_known_module __coq_plugin_name
+let _ = Mltop.add_known_module "coq-core.plugins.funind"
 
 # 31 "plugins/funind/g_indfun.mlg"
  
@@ -46,7 +45,7 @@ let pr_fun_ind_using_typed prc prlc _ opt_c =
 
 
 
-let (wit_fun_ind_using, fun_ind_using) = Tacentries.argument_extend ~name:"fun_ind_using" 
+let (wit_fun_ind_using, fun_ind_using) = Tacentries.argument_extend ~plugin:"coq-core.plugins.funind" ~name:"fun_ind_using" 
                                          {
                                          Tacentries.arg_parsing = Vernacextend.Arg_rules (
                                                                   [(Pcoq.Production.make
@@ -61,7 +60,7 @@ let (wit_fun_ind_using, fun_ind_using) = Tacentries.argument_extend ~name:"fun_i
                                                                    (Pcoq.Rule.next 
                                                                    (Pcoq.Rule.next 
                                                                    (Pcoq.Rule.stop)
-                                                                   ((Pcoq.Symbol.token (CLexer.terminal "using"))))
+                                                                   ((Pcoq.Symbol.token (Pcoq.terminal "using"))))
                                                                    ((Pcoq.Symbol.nterm constr_with_bindings)))
                                                                    (fun c _
                                                                    loc -> 
@@ -94,7 +93,7 @@ let (wit_fun_ind_using, fun_ind_using) = Tacentries.argument_extend ~name:"fun_i
                                          }
 let _ = (wit_fun_ind_using, fun_ind_using)
 
-let () = Tacentries.tactic_extend __coq_plugin_name "newfuninv" ~level:0 
+let () = Tacentries.tactic_extend "coq-core.plugins.funind" "newfuninv" ~level:0 
          [(Tacentries.TyML (Tacentries.TyIdent ("functional", Tacentries.TyIdent ("inversion", 
                                                               Tacentries.TyArg (
                                                               Extend.TUentry (Genarg.get_arg_tag wit_quantified_hypothesis), 
@@ -113,11 +112,10 @@ let () = Tacentries.tactic_extend __coq_plugin_name "newfuninv" ~level:0
 # 72 "plugins/funind/g_indfun.mlg"
  
 
-let pr_intro_as_pat _prc _ _ pat =
+let pr_intro_as_pat prc pat =
   match pat with
     | Some pat ->
-      spc () ++ str "as" ++ spc () ++ (* Miscprint.pr_intro_pattern prc  pat *)
-        str"<simple_intropattern>"
+      str "as" ++ spc () ++ Miscprint.pr_intro_pattern prc pat
     | None -> mt ()
 
 let out_disjunctive = CAst.map (function
@@ -126,24 +124,24 @@ let out_disjunctive = CAst.map (function
 
 
 
-let (wit_with_names, with_names) = Tacentries.argument_extend ~name:"with_names" 
+let (wit_with_names, with_names) = Tacentries.argument_extend ~plugin:"coq-core.plugins.funind" ~name:"with_names" 
                                    {
                                    Tacentries.arg_parsing = Vernacextend.Arg_rules (
                                                             [(Pcoq.Production.make
                                                               (Pcoq.Rule.stop)
                                                               (fun loc -> 
-# 89 "plugins/funind/g_indfun.mlg"
+# 91 "plugins/funind/g_indfun.mlg"
            None 
                                                                     ));
                                                             (Pcoq.Production.make
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
-                                                             ((Pcoq.Symbol.token (CLexer.terminal "as"))))
+                                                             ((Pcoq.Symbol.token (Pcoq.terminal "as"))))
                                                              ((Pcoq.Symbol.nterm simple_intropattern)))
                                                              (fun ipat _
                                                              loc -> 
-# 88 "plugins/funind/g_indfun.mlg"
+# 90 "plugins/funind/g_indfun.mlg"
                                            Some ipat 
                                                                     ))]);
                                    Tacentries.arg_tag = Some
@@ -157,22 +155,22 @@ let (wit_with_names, with_names) = Tacentries.argument_extend ~name:"with_names"
                                                            (wit_intro_pattern));
                                    Tacentries.arg_printer = ((fun env sigma -> 
                                                             
-# 87 "plugins/funind/g_indfun.mlg"
-                                                                      pr_intro_as_pat 
+# 88 "plugins/funind/g_indfun.mlg"
+                   fun prc _ _ -> pr_intro_as_pat (prc env sigma) 
+                                                            ), (fun env sigma -> 
+                                                            
+# 89 "plugins/funind/g_indfun.mlg"
+                    fun prc _ _ -> pr_intro_as_pat (prc env sigma) 
                                                             ), (fun env sigma -> 
                                                             
 # 87 "plugins/funind/g_indfun.mlg"
-                                                                      pr_intro_as_pat 
-                                                            ), (fun env sigma -> 
-                                                            
-# 87 "plugins/funind/g_indfun.mlg"
-                                                                      pr_intro_as_pat 
+               fun prc _ _ -> pr_intro_as_pat (fun c -> prc env sigma @@ snd @@ c env sigma) 
                                                             ));
                                    }
 let _ = (wit_with_names, with_names)
 
 
-# 92 "plugins/funind/g_indfun.mlg"
+# 94 "plugins/funind/g_indfun.mlg"
  
 
 let functional_induction b c x pat =
@@ -180,7 +178,7 @@ let functional_induction b c x pat =
 
 
 
-let () = Tacentries.tactic_extend __coq_plugin_name "newfunind" ~level:0 
+let () = Tacentries.tactic_extend "coq-core.plugins.funind" "newfunind" ~level:0 
          [(Tacentries.TyML (Tacentries.TyIdent ("functional", Tacentries.TyIdent ("induction", 
                                                               Tacentries.TyArg (
                                                               Extend.TUentry (Genarg.get_arg_tag wit_lconstr), 
@@ -190,14 +188,14 @@ let () = Tacentries.tactic_extend __coq_plugin_name "newfunind" ~level:0
                                                               Extend.TUentry (Genarg.get_arg_tag wit_with_names), 
                                                               Tacentries.TyNil))))), 
            (fun c princl pat ist -> 
-# 101 "plugins/funind/g_indfun.mlg"
+# 103 "plugins/funind/g_indfun.mlg"
     
-     (Extratactics.onSomeWithHoles
+     (Ltac_plugin.Internals.onSomeWithHoles
           (fun x -> functional_induction true c x pat) princl)
    
            )))]
 
-let () = Tacentries.tactic_extend __coq_plugin_name "snewfunind" ~level:0 
+let () = Tacentries.tactic_extend "coq-core.plugins.funind" "snewfunind" ~level:0 
          [(Tacentries.TyML (Tacentries.TyIdent ("soft", Tacentries.TyIdent ("functional", 
                                                         Tacentries.TyIdent ("induction", 
                                                         Tacentries.TyArg (
@@ -209,25 +207,25 @@ let () = Tacentries.tactic_extend __coq_plugin_name "snewfunind" ~level:0
                                                         Extend.TUentry (Genarg.get_arg_tag wit_with_names), 
                                                         Tacentries.TyNil)))))), 
            (fun cl princl pat ist -> 
-# 110 "plugins/funind/g_indfun.mlg"
+# 112 "plugins/funind/g_indfun.mlg"
       
        let c = match cl with
          | [] -> assert false
          | [c] -> c
          | c::cl -> EConstr.applist(c,cl)
        in
-       Extratactics.onSomeWithHoles (fun x -> functional_induction false c x pat) princl 
+       Ltac_plugin.Internals.onSomeWithHoles (fun x -> functional_induction false c x pat) princl 
            )))]
 
 
-# 119 "plugins/funind/g_indfun.mlg"
+# 121 "plugins/funind/g_indfun.mlg"
  
 
 let pr_constr_comma_sequence env sigma prc _ _ = prlist_with_sep pr_comma (prc env sigma)
 
 
 
-let (wit_constr_comma_sequence', constr_comma_sequence') = Tacentries.argument_extend ~name:"constr_comma_sequence'" 
+let (wit_constr_comma_sequence', constr_comma_sequence') = Tacentries.argument_extend ~plugin:"coq-core.plugins.funind" ~name:"constr_comma_sequence'" 
                                                            {
                                                            Tacentries.arg_parsing = 
                                                            Vernacextend.Arg_rules (
@@ -236,7 +234,7 @@ let (wit_constr_comma_sequence', constr_comma_sequence') = Tacentries.argument_e
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm constr)))
                                                              (fun c loc -> 
-# 129 "plugins/funind/g_indfun.mlg"
+# 131 "plugins/funind/g_indfun.mlg"
                      [c] 
                                                                     ));
                                                            (Pcoq.Production.make
@@ -245,11 +243,11 @@ let (wit_constr_comma_sequence', constr_comma_sequence') = Tacentries.argument_e
                                                             (Pcoq.Rule.next 
                                                             (Pcoq.Rule.stop)
                                                             ((Pcoq.Symbol.nterm constr)))
-                                                            ((Pcoq.Symbol.token (CLexer.terminal ","))))
+                                                            ((Pcoq.Symbol.token (Pcoq.terminal ","))))
                                                             (Pcoq.Symbol.self))
                                                             (fun l _ c loc ->
                                                             
-# 128 "plugins/funind/g_indfun.mlg"
+# 130 "plugins/funind/g_indfun.mlg"
                                                    c::l 
                                                             ))]);
                                                            Tacentries.arg_tag = 
@@ -268,47 +266,47 @@ let (wit_constr_comma_sequence', constr_comma_sequence') = Tacentries.argument_e
                                                            Tacentries.arg_printer = 
                                                            ((fun env sigma -> 
                                                            
-# 127 "plugins/funind/g_indfun.mlg"
+# 129 "plugins/funind/g_indfun.mlg"
                pr_constr_comma_sequence env sigma 
                                                            ), (fun env sigma -> 
                                                            
-# 127 "plugins/funind/g_indfun.mlg"
+# 129 "plugins/funind/g_indfun.mlg"
                pr_constr_comma_sequence env sigma 
                                                            ), (fun env sigma -> 
                                                            
-# 127 "plugins/funind/g_indfun.mlg"
+# 129 "plugins/funind/g_indfun.mlg"
                pr_constr_comma_sequence env sigma 
                                                            ));
                                                            }
 let _ = (wit_constr_comma_sequence', constr_comma_sequence')
 
 
-# 132 "plugins/funind/g_indfun.mlg"
+# 134 "plugins/funind/g_indfun.mlg"
  
 
 let pr_auto_using env sigma prc _prlc _prt = Pptactic.pr_auto_using (prc env sigma)
 
 
 
-let (wit_auto_using', auto_using') = Tacentries.argument_extend ~name:"auto_using'" 
+let (wit_auto_using', auto_using') = Tacentries.argument_extend ~plugin:"coq-core.plugins.funind" ~name:"auto_using'" 
                                      {
                                      Tacentries.arg_parsing = Vernacextend.Arg_rules (
                                                               [(Pcoq.Production.make
                                                                 (Pcoq.Rule.stop)
                                                                 (fun loc -> 
-# 142 "plugins/funind/g_indfun.mlg"
+# 144 "plugins/funind/g_indfun.mlg"
            [] 
                                                                     ));
                                                               (Pcoq.Production.make
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
-                                                               ((Pcoq.Symbol.token (CLexer.terminal "using"))))
+                                                               ((Pcoq.Symbol.token (Pcoq.terminal "using"))))
                                                                ((Pcoq.Symbol.nterm constr_comma_sequence')))
                                                                (fun l _
                                                                loc -> 
                                                                
-# 141 "plugins/funind/g_indfun.mlg"
+# 143 "plugins/funind/g_indfun.mlg"
                                              l 
                                                                ))]);
                                      Tacentries.arg_tag = Some
@@ -322,26 +320,25 @@ let (wit_auto_using', auto_using') = Tacentries.argument_extend ~name:"auto_usin
                                                              (wit_constr));
                                      Tacentries.arg_printer = ((fun env sigma -> 
                                                               
-# 140 "plugins/funind/g_indfun.mlg"
+# 142 "plugins/funind/g_indfun.mlg"
                pr_auto_using env sigma 
                                                               ), (fun env sigma -> 
                                                               
-# 140 "plugins/funind/g_indfun.mlg"
+# 142 "plugins/funind/g_indfun.mlg"
                pr_auto_using env sigma 
                                                               ), (fun env sigma -> 
                                                               
-# 140 "plugins/funind/g_indfun.mlg"
+# 142 "plugins/funind/g_indfun.mlg"
                pr_auto_using env sigma 
                                                               ));
                                      }
 let _ = (wit_auto_using', auto_using')
 
 
-# 145 "plugins/funind/g_indfun.mlg"
+# 147 "plugins/funind/g_indfun.mlg"
  
 
 module Vernac = Pvernac.Vernac_
-module Tactic = Pltac
 
 let (wit_function_fix_definition : Vernacexpr.fixpoint_expr Loc.located Genarg.uniform_genarg_type) =
   Genarg.create_arg "function_fix_definition"
@@ -354,20 +351,21 @@ let function_fix_definition =
 let _ = let () = assert (Pcoq.Entry.is_empty function_fix_definition)
         in
         let () =
-        Pcoq.grammar_extend function_fix_definition
+        Egramml.grammar_extend ~plugin_uid:("coq-core.plugins.funind", "g_indfun.mlg:0")
+        function_fix_definition
         (Pcoq.Fresh
         (Gramlib.Gramext.First, [(None, None,
                                  [Pcoq.Production.make
                                   (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                   ((Pcoq.Symbol.nterm Vernac.fix_definition)))
                                   (fun g loc -> 
-# 162 "plugins/funind/g_indfun.mlg"
+# 163 "plugins/funind/g_indfun.mlg"
                                        Loc.tag ~loc g 
                                                 )])]))
         in ()
 
 
-# 167 "plugins/funind/g_indfun.mlg"
+# 168 "plugins/funind/g_indfun.mlg"
  
 
 let () =
@@ -383,7 +381,7 @@ let is_proof_termination_interactively_checked recsl =
 
 let classify_as_Fixpoint recsl =
  Vernac_classifier.classify_vernac
-    (Vernacexpr.(CAst.make @@ { control = []; attrs = []; expr = VernacFixpoint(NoDischarge, List.map snd recsl)}))
+    (Vernacexpr.(CAst.make @@ { control = []; attrs = []; expr = VernacSynPure (VernacFixpoint(NoDischarge, List.map snd recsl))}))
 
 let classify_funind recsl =
   match classify_as_Fixpoint recsl with
@@ -399,21 +397,21 @@ let is_interactive recsl =
 
 
 
-let () = Vernacextend.vernac_extend ~command:"Function"  ?entry:None 
+let () = Vernacextend.static_vernac_extend ~plugin:(Some "coq-core.plugins.funind") ~command:"Function"  ?entry:None 
          [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Function", 
                                      Vernacextend.TyNonTerminal (Extend.TUlist1sep (
                                                                  Extend.TUentry (Genarg.get_arg_tag wit_function_fix_definition), "with"), 
                                      Vernacextend.TyNil)), (let coqpp_body recsl
                                                            () = 
-# 210 "plugins/funind/g_indfun.mlg"
+# 211 "plugins/funind/g_indfun.mlg"
         
-    let warn = "-unused-pattern-matching-variable,-matching-variable,-non-recursive" in
+    let warn = "-unused-pattern-matching-variable,-non-recursive" in
     if is_interactive recsl then
-      Vernacextend.VtOpenProof (fun () ->
+      Vernactypes.vtopenproof (fun () ->
           CWarnings.with_warn warn
             Gen_principle.do_generate_principle_interactive (List.map snd recsl))
     else
-      Vernacextend.VtDefault (fun () ->
+      Vernactypes.vtdefault (fun () ->
           CWarnings.with_warn warn
             Gen_principle.do_generate_principle (List.map snd recsl))
   
@@ -422,12 +420,12 @@ let () = Vernacextend.vernac_extend ~command:"Function"  ?entry:None
                                                            -> coqpp_body recsl
                                                            (Attributes.unsupported_attributes atts)), Some 
          (fun recsl -> 
-# 209 "plugins/funind/g_indfun.mlg"
+# 210 "plugins/funind/g_indfun.mlg"
          classify_funind recsl 
          )))]
 
 
-# 223 "plugins/funind/g_indfun.mlg"
+# 224 "plugins/funind/g_indfun.mlg"
  
 
 let pr_fun_scheme_arg (princ_name,fun_name,s) =
@@ -437,7 +435,7 @@ let pr_fun_scheme_arg (princ_name,fun_name,s) =
 
 
 
-let (wit_fun_scheme_arg, fun_scheme_arg) = Vernacextend.vernac_argument_extend ~name:"fun_scheme_arg" 
+let (wit_fun_scheme_arg, fun_scheme_arg) = Vernacextend.vernac_argument_extend ~plugin:"coq-core.plugins.funind" ~name:"fun_scheme_arg" 
                                            {
                                            Vernacextend.arg_parsing = 
                                            Vernacextend.Arg_rules ([(
@@ -450,12 +448,12 @@ let (wit_fun_scheme_arg, fun_scheme_arg) = Vernacextend.vernac_argument_extend ~
                                                                    (Pcoq.Rule.next 
                                                                    (Pcoq.Rule.next 
                                                                    (Pcoq.Rule.stop)
-                                                                   ((Pcoq.Symbol.nterm ident)))
-                                                                   ((Pcoq.Symbol.token (CLexer.terminal ":="))))
-                                                                   ((Pcoq.Symbol.token (CLexer.terminal "Induction"))))
-                                                                   ((Pcoq.Symbol.token (CLexer.terminal "for"))))
+                                                                   ((Pcoq.Symbol.nterm identref)))
+                                                                   ((Pcoq.Symbol.token (Pcoq.terminal ":="))))
+                                                                   ((Pcoq.Symbol.token (Pcoq.terminal "Induction"))))
+                                                                   ((Pcoq.Symbol.token (Pcoq.terminal "for"))))
                                                                    ((Pcoq.Symbol.nterm reference)))
-                                                                   ((Pcoq.Symbol.token (CLexer.terminal "Sort"))))
+                                                                   ((Pcoq.Symbol.token (Pcoq.terminal "Sort"))))
                                                                    ((Pcoq.Symbol.nterm sort_family)))
                                                                    (fun s _
                                                                    fun_name _
@@ -463,19 +461,19 @@ let (wit_fun_scheme_arg, fun_scheme_arg) = Vernacextend.vernac_argument_extend ~
                                                                    princ_name
                                                                    loc -> 
                                                                    
-# 234 "plugins/funind/g_indfun.mlg"
-                                                                                              (princ_name,fun_name,s) 
+# 235 "plugins/funind/g_indfun.mlg"
+                                                                                                 (princ_name.CAst.v,fun_name,s) 
                                                                    ))]);
                                            Vernacextend.arg_printer = fun env sigma -> 
                                            
-# 233 "plugins/funind/g_indfun.mlg"
+# 234 "plugins/funind/g_indfun.mlg"
              pr_fun_scheme_arg 
                                            ;
                                            }
 let _ = (wit_fun_scheme_arg, fun_scheme_arg)
 
 
-# 237 "plugins/funind/g_indfun.mlg"
+# 238 "plugins/funind/g_indfun.mlg"
  
 
 let warning_error names e =
@@ -492,15 +490,15 @@ let warning_error names e =
 
 
 
-let () = Vernacextend.vernac_extend ~command:"NewFunctionalScheme"  ?entry:None 
+let () = Vernacextend.static_vernac_extend ~plugin:(Some "coq-core.plugins.funind") ~command:"NewFunctionalScheme"  ?entry:None 
          [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Functional", 
                                      Vernacextend.TyTerminal ("Scheme", 
                                      Vernacextend.TyNonTerminal (Extend.TUlist1sep (
                                                                  Extend.TUentry (Genarg.get_arg_tag wit_fun_scheme_arg), "with"), 
                                      Vernacextend.TyNil))), (let coqpp_body fas
-                                                            () = Vernacextend.VtDefault (fun () -> 
+                                                            () = Vernactypes.vtdefault (fun () -> 
                                                                  
-# 257 "plugins/funind/g_indfun.mlg"
+# 258 "plugins/funind/g_indfun.mlg"
       begin
         try
           Gen_principle.build_scheme fas
@@ -531,37 +529,37 @@ let () = Vernacextend.vernac_extend ~command:"NewFunctionalScheme"  ?entry:None
                                                             -> coqpp_body fas
                                                             (Attributes.unsupported_attributes atts)), Some 
          (fun fas -> 
-# 255 "plugins/funind/g_indfun.mlg"
+# 256 "plugins/funind/g_indfun.mlg"
         Vernacextend.(VtSideff(List.map pi1 fas, VtLater)) 
          )))]
 
-let () = Vernacextend.vernac_extend ~command:"NewFunctionalCase"  ?entry:None 
+let () = Vernacextend.static_vernac_extend ~plugin:(Some "coq-core.plugins.funind") ~command:"NewFunctionalCase"  ?entry:None 
          [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Functional", 
                                      Vernacextend.TyTerminal ("Case", 
                                      Vernacextend.TyNonTerminal (Extend.TUentry (Genarg.get_arg_tag wit_fun_scheme_arg), 
                                      Vernacextend.TyNil))), (let coqpp_body fas
-                                                            () = Vernacextend.VtDefault (fun () -> 
+                                                            () = Vernactypes.vtdefault (fun () -> 
                                                                  
-# 288 "plugins/funind/g_indfun.mlg"
+# 289 "plugins/funind/g_indfun.mlg"
        Gen_principle.build_case_scheme fas 
                                                                  ) in fun fas
                                                             ?loc ~atts ()
                                                             -> coqpp_body fas
                                                             (Attributes.unsupported_attributes atts)), Some 
          (fun fas -> 
-# 287 "plugins/funind/g_indfun.mlg"
+# 288 "plugins/funind/g_indfun.mlg"
        Vernacextend.(VtSideff([pi1 fas], VtLater)) 
          )))]
 
-let () = Vernacextend.vernac_extend ~command:"GenerateGraph" ~classifier:(fun _ -> Vernacextend.classify_as_query) ?entry:None 
+let () = Vernacextend.static_vernac_extend ~plugin:(Some "coq-core.plugins.funind") ~command:"GenerateGraph" ~classifier:(fun _ -> Vernacextend.classify_as_query) ?entry:None 
          [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Generate", 
                                      Vernacextend.TyTerminal ("graph", 
                                      Vernacextend.TyTerminal ("for", 
                                      Vernacextend.TyNonTerminal (Extend.TUentry (Genarg.get_arg_tag wit_reference), 
                                      Vernacextend.TyNil)))), (let coqpp_body c
-                                                             () = Vernacextend.VtDefault (fun () -> 
+                                                             () = Vernactypes.vtdefault (fun () -> 
                                                                   
-# 294 "plugins/funind/g_indfun.mlg"
+# 295 "plugins/funind/g_indfun.mlg"
     Gen_principle.make_graph (Smartlocate.global_with_alias c) 
                                                                   ) in fun c
                                                              ?loc ~atts ()
