@@ -25,7 +25,11 @@ let () = Vernacextend.static_vernac_extend ~plugin:(Some "coq-core.plugins.ltac"
                                                             ) in fun cl
                                                             ?loc ~atts ()
                                                             -> coqpp_body cl
-                                                            (Attributes.parse tc_transparency_locality atts)), None))]
+                                                            (Attributes.parse 
+                                                            
+# 24 "plugins/ltac/g_class.mlg"
+                 Attributes.hint_locality
+                                                             atts)), None))]
 
 let () = Vernacextend.static_vernac_extend ~plugin:(Some "coq-core.plugins.ltac") ~command:"Typeclasses_Rigid_Settings" ~classifier:(fun _ -> Vernacextend.classify_as_sideeff) ?entry:None 
          [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Typeclasses", 
@@ -43,7 +47,11 @@ let () = Vernacextend.static_vernac_extend ~plugin:(Some "coq-core.plugins.ltac"
                                                             ) in fun cl
                                                             ?loc ~atts ()
                                                             -> coqpp_body cl
-                                                            (Attributes.parse tc_transparency_locality atts)), None))]
+                                                            (Attributes.parse 
+                                                            
+# 31 "plugins/ltac/g_class.mlg"
+                 Attributes.hint_locality
+                                                             atts)), None))]
 
 
 # 37 "plugins/ltac/g_class.mlg"
