@@ -88,7 +88,7 @@ let _ = let () = assert (Pcoq.Entry.is_empty withtac) in
 # 73 "plugins/ltac/g_obligations.mlg"
                                                                                 
           let typ = mkAppC (sigref loc, [mkLambdaC ([id], default_binder_kind, t, c)]) in
-          [CLocalAssum ([id], default_binder_kind, typ)] 
+          [CLocalAssum ([id], None, default_binder_kind, typ)] 
                                                        )]))
         in ()
 
@@ -331,8 +331,10 @@ let () = Vernacextend.static_vernac_extend ~plugin:(Some "coq-core.plugins.ltac"
           (Tacintern.glob_tactic t);
   
                     ) in fun t
-         ?loc ~atts () -> coqpp_body t
-         (Attributes.parse Tactic_option.tac_option_locality atts)), None))]
+         ?loc ~atts () -> coqpp_body t (Attributes.parse 
+# 132 "plugins/ltac/g_obligations.mlg"
+                 Tactic_option.tac_option_locality
+          atts)), None))]
 
 
 # 139 "plugins/ltac/g_obligations.mlg"
