@@ -312,7 +312,7 @@ let _ = let () =
                                             ((Pcoq.Symbol.nterml term ("99"))))
                             (fun c _ loc -> 
 # 124 "plugins/ssr/ssrvernac.mlg"
-        [CLocalAssum ([CAst.make ~loc Anonymous], Default Explicit, c)] 
+        [CLocalAssum ([CAst.make ~loc Anonymous], None, Default Explicit, c)] 
                                             )]))
         in ()
 
@@ -360,7 +360,11 @@ let () = Vernacextend.static_vernac_extend ~plugin:(Some "coq-core.plugins.ssref
                                                             ) in fun fl
                                                             ?loc ~atts ()
                                                             -> coqpp_body fl
-                                                            (Attributes.parse Attributes.locality atts)), None))]
+                                                            (Attributes.parse 
+                                                            
+# 168 "plugins/ssr/ssrvernac.mlg"
+                   Attributes.locality
+                                                             atts)), None))]
 
 let _ = let () =
         Egramml.grammar_extend ~plugin_uid:("coq-core.plugins.ssreflect", "ssrvernac.mlg:8")
